/*
**	File:	
**		ImageCanvas.java
**
**	Description:	
**		Object to hold the loaded JPEG or GIF image,
**		to handle the mouse events, and to compute the 
**		area of the traced region.
**
**	Author:	
**		Paul B. Davis
*******************************************************************************
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

public class ImageCanvas extends JPanel {
  private Vector points = null;
  private double area = 0.0;
  private Point sp = null;
  private Point ep = null;
  private boolean first_click = true;
  private ImageIcon imageicon = null;
  private JLabel image = null;
  private double scale = 1.0;
  private Main parent = null;

  public ImageCanvas(final Main p) {
    	parent = p;	
	points = new Vector();
	ep = new Point();
	sp = new Point();
	image = new JLabel();

	addMouseMotionListener(new MouseMotionAdapter() {
	  public void mouseDragged(MouseEvent e ) {
	    points.addElement(new Point(e.getX(),e.getY()));
	  }
	});
		
	addMouseListener(new MouseAdapter() {
          public void mousePressed(MouseEvent evt) {
	    if((evt.getModifiers()&InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK) {
	      if((evt.getClickCount() == 2) && first_click) {
	        first_click = false;
		sp.setLocation(evt.getX(), evt.getY());
	      } else if((evt.getClickCount() == 2) && !first_click) {
		first_click = true;
		ep.setLocation(evt.getX(),evt.getY());
		try {
		  String s = JOptionPane.showInputDialog("Equals how many units?");
		  Double ds = new Double(s);
		  scale = (ds.doubleValue())/
			  (Math.sqrt( Math.pow((sp.getX()-ep.getX()),2)
		 	  + Math.pow((sp.getY()-ep.getY()),2)));
		} catch(NullPointerException npe) {
		  // user has entered nothing or pressed cancel
		} catch(NumberFormatException nfe) {
		  JOptionPane.showMessageDialog(
		   parent, "Invalid Input. Please try again.",
		   "ERROR", JOptionPane.ERROR_MESSAGE
		  );
		} catch(Exception e) {
		  JOptionPane.showMessageDialog(
		   parent, e.toString(),
		   "Error setting scale.",JOptionPane.ERROR_MESSAGE
		  );
		}		
	      } else { /* do nothing */ }	
	    } else { /* do nothing */ }
	  }
        });
	add(image);
  }	
  
  public void setImage(final String ipath) {
    imageicon = new ImageIcon(ipath);
    image.setIcon(imageicon);
  }

  public void clearAreaOnly(final Component p, final boolean msg) {
    points.removeAllElements();
    area = 0.0;
    if(msg) {
      JOptionPane.showMessageDialog(
        p, "Area has been reset!",
        "AREA RESET", JOptionPane.INFORMATION_MESSAGE
      );	
    }
  }

  public void clearAll(final Component p) {
    points.removeAllElements();
    area = 0.0;	
    scale = 1.0;
    JOptionPane.showMessageDialog(
      p, "Area and scale have been reset!",
      "TOTAL RESET", JOptionPane.INFORMATION_MESSAGE
    );
  }

  public void resetScale(final Component p) {
    scale = 1.0;
    JOptionPane.showMessageDialog(
      p, "Scale has been reset!",
      "SCALE RESET", JOptionPane.INFORMATION_MESSAGE
    );
  }
  
  public void viewScale(final Component p) {
    JOptionPane.showMessageDialog(
      p, "Scale is: " + scale,
      "Current Scale", JOptionPane.INFORMATION_MESSAGE
    );
  }

  public void doAreaComputation() {
    if( points.size() <= 2 ) { area = 0.0; }
    else { 
      // Make the first point the last point.
      // This closes the region.
      points.addElement((Point)points.elementAt(0));
      computeArea(); 
    }
    JOptionPane.showMessageDialog(
      parent, "Area = " + area + "\nScale = " + scale,
      "Computed Area", JOptionPane.INFORMATION_MESSAGE
    );
  }
	
  private void computeArea() {
   int dimension = points.size();
   double [] x = new double[dimension];
   double [] y = new double[dimension];

   for(int i = 0; i < points.size(); i++) {
     x[i]=(((Point)points.elementAt(i)).getX());
     y[i]=(((Point)points.elementAt(i)).getY());
   }

   // Apply the formula from theorem 1.2
   for(int i = 1; i <= x.length-1; i++ ) {
     area+=(y[i-1]+y[i+1-1])*(x[i+1-1]-x[i-1]);	
   }
   // Take the absolute value and multiply by 1/2
   area=((area<0)?-area:area)/2;
   // Adjust for the scale of the current map.
   area*=Math.pow(scale,2);
  }
}	
