/*
**	File:	
**		Main.java
**
**	Description:
**		This is the main class to drive the application.
**
**	Author:
**		Paul B. Davis
*******************************************************************************
*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.io.*;

public class Main extends JFrame implements ActionListener {
  private final String states[] = {
    "Alabama","Alaska","Arizona","Arkansas","California", 
    "Colorado","Connecticut","Delaware","Florida","Georgia", 
    "Hawaii","Idaho","Illinois","Indiana","Iowa", 
    "Kansas","Kentucky","Louisiana","Maine","Maryland", 
    "Massachusetts","Michigan","Minnesota","Mississippi","Missouri", 
    "Montana","Nebraska","Nevada","New Hampshire","New Jersey", 	
    "New Mexico","New York","North Carolina","North Dakota","Ohio", 
    "Oklahoma","Oregon","Pensylvania","Rhode Island","South Carolina", 
    "South Dakota","Tennesse","Texas","Utah","Vermont",
    "Virginia","Washington","West Virginia","Wisconsin","Wyoming" 
  };
	
  private final String areas[] = {
    "52,419.02 mi","663,267.26 mi","113,998.30 mi","53,178.62 mi","163,695.57 mi", 
    "104,093.57 mi","5,543.33 mi","2,489.27 mi","65,754.59 mi","59,424.77 mi", 
    "10,930.98 mi","83,570.08 mi","57,914.38 mi","36,417.73 mi","56,271.55 mi",
    "82,276.84 mi","40,409.02 mi","51,839.70 mi","35,384.65 mi","12,406.68 mi", 
    "10,554.57 mi","96,716.11 mi","86,938.87 mi","48,430.19 mi","69,704.31 mi", 
    "147,042.40 mi","77,353.73 mi","110,560.71 mi","9,349.94 mi","8,721.30 mi", 
    "121,589.48 mi","54,556.00 mi","53,818.51 mi","70,699.79 mi","44,824.90 mi",
    "69,898.19 mi","98,380.64 mi","46,055.24 mi","1,545.05 mi","32,020.20 mi", 
    "77,116.49 mi","42,143.27 mi","268,580.82 mi","84,898.83 mi","9,614.26 mi", 
    "42,774.20 mi","71,299.64 mi","24,229.76 mi","65,497.82 mi","97,813.56 mi"
  }; 

  private final int WINDOW_HEIGHT = 512;
  private final int WINDOW_WIDTH = 512;
  
  // Indices into the menuItems array
  private final int OPEN_IMAGE_MI = 0;
  private final int COMPUTE_AREA_MI = 1;
  private final int ACTUAL_AREA_MI = 2;
  private final int VIEW_SCALE_MI = 3;
  private final int RESET_ALL_MI = 4;
  private final int RESET_AREA_MI = 5;
  private final int RESET_SCALE_MI = 6;
  private final int EXIT_MI = 7;

  private ImageCanvas panel = null;
  private Container contentPane = null;
  private JFileChooser chooser = null;
	
  private JMenuItem [] menuItems = {
    new JMenuItem("Open New Image ..."),
    new JMenuItem("Compute Area"),
    new JMenuItem("Area by State"),
    new JMenuItem("View Scale"),
    new JMenuItem("Reset All"),
    new JMenuItem("Reset Area"),
    new JMenuItem("Reset Scale"),
    new JMenuItem("Exit")
  };

  // Constructor
  public Main() {
    super("Area Finder");
    addWindowListener(new WindowAdapter() {
    public void windowClosing(WindowEvent e) {
      System.exit(0);
    }});
    this.setSize(WINDOW_HEIGHT,WINDOW_WIDTH);
    this.setResizable(true);
    panel = new ImageCanvas(this);
    contentPane = getContentPane();
    chooser = new JFileChooser(".");
    chooser.setFileFilter( new GIFandJPEG_FileFilter() );
    JMenuBar menuBar = new JMenuBar();
    JMenu controlMenu = new JMenu("Menu");
    for(int i=0; i<menuItems.length; i++){
      controlMenu.add(menuItems[i]);
      menuItems[i].addActionListener(this);
      if((i==menuItems.length-2) || (i==0) || (i==3)) {
	controlMenu.addSeparator();
      }
    }
    JScrollPane scrollpane = new JScrollPane(panel);
    scrollpane.setPreferredSize(new Dimension(WINDOW_HEIGHT,WINDOW_WIDTH));	
    contentPane.add(scrollpane);
    menuBar.add(controlMenu);
    setJMenuBar(menuBar);
  }

 public void actionPerformed(ActionEvent e) {
   Object source = e.getSource();
   if(source == menuItems[OPEN_IMAGE_MI]) {
     if(JFileChooser.APPROVE_OPTION == chooser.showOpenDialog(this)) {
       String s = chooser.getSelectedFile().getAbsolutePath();
       panel.setImage(s);
     }
   } else if(source == menuItems[ACTUAL_AREA_MI]) {
     String s = (String)JOptionPane.showInputDialog(
       Main.this,
       "Choose a state",
       "Actual Area",
       JOptionPane.INFORMATION_MESSAGE,
       null,
       states,	
       states[0]
     );
     if(s != null) {
       int i = 0;
       while(states[i] != s) {i++;}
       JOptionPane.showMessageDialog(
         Main.this,
         areas[i],
         "Area for "+states[i],
         JOptionPane.INFORMATION_MESSAGE
       );
     }
   } else if(source == menuItems[RESET_AREA_MI]) {
     panel.clearAreaOnly(this,true);
   } else if(source == menuItems[RESET_SCALE_MI]) {
     panel.resetScale(this);
   } else if(source == menuItems[COMPUTE_AREA_MI]) {
     panel.doAreaComputation(); 
     panel.clearAreaOnly(this,false);
   } else if(source == menuItems[VIEW_SCALE_MI]) {
     panel.viewScale(this);
   } else if(source == menuItems[RESET_ALL_MI]) {
     panel.clearAll(this);
   } else if(source == menuItems[EXIT_MI]) {
     System.exit(0);
   } else { /* do nothing */ }
 }

 public static void main(String [] args) {
   JFrame jf = new Main();
   jf.show();
 }
}	

// file chooser filter so that only jpeg and gif images are seen
class GIFandJPEG_FileFilter extends javax.swing.filechooser.FileFilter {
  public boolean accept(File f) {
    return (f.getName().toLowerCase().endsWith(".jpg") ||
            f.getName().toLowerCase().endsWith(".jpeg") ||
            f.getName().toLowerCase().endsWith(".gif") ||
            f.isDirectory());
  }
  public String getDescription(){ return "JPEG & GIF Images"; }
}
